(* *************************
   Unit uGridPanel component TGridPanel

   Autor: Jan Tungli
   email: jan.tungli@seznam.cz ,
   web: www.host.sk/tungli

   Komponent pre zobrazenie planovanych uloh v kalendary.
   GridPanel sluzi na zobrazenie udajov tak ze jedna bunka (grid) moze obsahovat
   x pocet poloziek.
   Polozky su v gride a skladaju sa z ikonky 16x16 (image) a pod nim text (label).

   Predpokladane pouzitie:
     Doporuceny font: "Ariel Narrow size 8"
     GridPanel sa polozi primo na ScrollPanel alebo na ScrollBox.
     GridPanel podla potreby modifikuje svoj rozmer.
     Pri focusnuti polozky pomocou metod
         LabelSetFocus alebo FocusByTagId sa automaticky zviditelni focusnuta
         polozka - posunutim scrollbaru v ScrolBoxu (ak je to potrebne).
     Ak focusnutie nebudete vyuzivat tak je mozne pouzit GridPanel ako Panel,
     potom netreba polozit na ScrollBox.


    Dolezite vlastnosti:
     CountColums  - pocet sloupcov (X)
     CountRows    - pocet riadkov  (Y)
     CountItems   - kolko poloziek sa ma zmestit do jednej bunky (vodorovne)
     GridDouble   - dvojposhodova bunka
     WallPaper    - obrazok do pozadia
     ActFocus     - polozka na ktorom je focus (read olny)
     ImageList    - na napojenie ImageListu
     ImageShow    - ci am zobrazit ikonku alebo nie
     TextCount    - pocet riadkov na text

    Dolezite procedury:
     procedure DataRefresh; - zobrazenie udajov (na zaciatku)
     procedure DrawGrid(x,y:word);  - zobrazi jednu bunku
     procedure LabelFocus(pLabel:TLabel; pMove:boolean=false);
               skor je to interna procedura, ktora dany label oznaci
               ako by mal focus a ak pMove je true tak nastavy aby polozka
               bola aj vyditelna
     procedure LabelSetFocus(x,y,index:integer);
               nastavy polozku vyditelnou a da do focusu
     function  FocusByTagId(pDbId:integer):boolean;
               tiez nastavy polozku vyditelnou a da do focusu, ale polozka je tu
               identifikovana podla databazoveho ID zaznamu , ktora je zapisana
               do TAG vlastnosti polozky.
     procedure ClearAll; - odstrani vsetky bunky



    Dolezita udalost je :
      OnGetData (Sender: TObject;
                 var Text:string;
                 var ImageIndex,ID:integer;
                 var Ok:boolean;
                 X,Y,Index:word;
                 Test:boolean=false)

      Text -text, ktora sa ma zobrazit - viac riadkov je mozne zadat s +#13#10
      Imageindex -ktory image z ImegeListu ma vykreslit 0 az n
      ID   -je mozne zapisat databazovy ID zaznamu
            tuto hodnotu zapise do tag vlastnosti polozky (label a image)
      Ok   -najdolezitejsi, ci posielame data do danej bunky (true)
            alebo uz nemame udaje (false)
      X,Y - su koordinaty bunky (logicke koordinaty)
            X  <0 ; CountCols-1>
            Y  <0 ; CountRows-1>
      ItemNo - vyjadruje ze kolikata je to polozka v bunk ItemNo <1..CountItems>
      Test  -ak je true tak GridPanel chce iba vediet ci este su dalsie polozky
             pre bunku. Ak Ok nastavime na false tak to znamena ze nie su.
             Toto je dobre na to aby pri zapisu poslednej polozky ktora sa este
             zmesti do bunky na koniec textu zobrazil znak ">" na upozornenie
             ze vsetky polozky sa nezmestili dobunky (do gridu).

priklad:
procedure TForm1.GridPanel1GetData(Sender: TObject; var text: String;
  var ImageIndex, TagID: Integer; var Ok: Boolean; x, y, ItemNo: Word; test: Boolean);
begin
  if not Table1.eof then begin
     Text:=Table1.FieldByName('Name').asString+#13#10+
           Table1.FieldByName('capital').asString+#13#10+
           Table1.FieldByName('Continent').asString+#13#10+
           Table1.FieldByName('Area').asString;
     ImageIndex:=0;
     Ok:=true;
     if not Test then Table1.Next;
  end else Ok:=false;
end;


   ************************** *)


unit uGridPanel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls,StdCtrls,uScrollPanel;

 const
    UM_labelDblClick=WM_User+106;

type
  TGridPanelGetData=procedure (Sender: TObject; var pText,pHint:string; var ImageIndex,TagID,TimeSize:integer; var Ok:boolean; var pColor:TColor; X,Y,ItemNo:word; Test:boolean=false) of object;
  TGridPanelGetBandData=procedure (Sender: TObject; var Text1,Text2,Hint:string;var Color1,Color2:Tcolor; grid:word) of object;
  TGridPanelDrawText=procedure (Sender: TObject; pLabel:TComponent; X,Y,ItemNo:word) of object;
  TGridItem=record
               pText,pHint:string;
               pID,pNewID,px,py,pind,pTimeSize,pImageIndex:integer;
               pColor:TColor;
               pOD:TDateTime;
               pODT:integer;
               pUpdateMode:integer;       //1,0,-1
               pLabel,pImage:TComponent;       //objekty
             end;

type
  TELABEL = class(TLabel)
  private
    fTimeSize:integer;
    fTimeLine:integer;
    fInd,fx,fy:integer;
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
  published
    property TimeSize:integer read fTimeSize write fTimeSize;
    property TimeLine:integer read fTimeLine write fTimeLine;
    property MyIndex:integer read fInd write fInd;
    property MyX:integer read fx write fx;
    property MyY:integer read fy write fy;
    { Published declarations }
  end;

  TGridPanel = class(TPanel)
  private
    FWallpaper: TPicture;
    fXCnt,fYCnt,fECnt,fTX,fx,fy:word;
    fTCnt,fLM,fCM:byte;
    fTextColor:Tcolor;
    fGetData:TGridPanelGetData;
    fDrawText:TGridPanelDrawText;
    fNewSize:TNotifyEvent;
    fBeforeRefresh,fAfterRefresh:TNotifyEvent;
    fFocusChange,fLabelClick,fLabelDblClick,fImageClick,fImageDblClick,fLeftLabelDblClick:TnotifyEvent;
    fDouble,fImageShow,fTrans,fCenter:boolean;
    fIMG1:TImage;
    fImageList:TImageList;
    fBevelSize:integer;  //pomocna
    fFocusColor,fFocusFontColor:Tcolor;

    fActFocus:TELabel;
    fActColor,fActFontColor:TColor;

    fGetTop,fGetLeft:TGridPanelGetBandData;
    fBandTop,fBandLeft:word;
    fTimeLine,fTimeLineExtra:byte;
    fTimeTrans:boolean;

    procedure SetfXCnt(pCnt:word);
    procedure SetfYCnt(pCnt:word);
    procedure SetfECnt(pCnt:word);
    procedure SetfTX(pWidth:word);
    procedure SetfTCnt(pCnt:byte);
    procedure SetfLM(pCnt:byte);
    procedure SetfCM(pCnt:byte);
    procedure SetfImageShow(pB:boolean);
    procedure SetfDouble(pB:boolean);
    procedure NewSize;
    procedure WallpaperChanged(Sender: TObject);
    procedure SetWallpaper(Value: TPicture);
    procedure SetfBandTop(pA:word);
    procedure SetfBandLeft(pA:word);
    procedure SetfTimeLine(pA:byte);
    procedure SetfTimeLineExtra(pA:byte);

    procedure LabelClick(Sender: TObject);
    procedure ImageClick(Sender: TObject);
    procedure LabelDblClick(Sender: TObject);
    procedure ImageDblClick(Sender: TObject);
    procedure LeftLabelDblClick(Sender: TObject);

    procedure DrawLeft;
    procedure DrawTop;
    procedure DrawLeftGrid(grid:word);
    procedure DrawTopGrid(grid:word);
    function  GetLabelX(x,y,mi:integer):integer;
    function  GetLabelY(x,y,mi:integer):integer;
    function  GetImageX(x,y,mi:integer):integer;
    procedure Proc_labelDblClick(var Msg: TMessage); message UM_LabelDblClick;
    function  GetImageY(x,y,mi:integer):integer;

    { Private declarations }
  protected
    procedure Paint; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure CreateItem(var G:TGridItem);
    { Protected declarations }
  public
     Constructor Create(AOwner: TComponent); override;
     Destructor Destroy; override;
    { Public declarations }
  published
     property BandTopHeight:word read fBandTop write SetfBandTop;
     property BandLeftWidth:word read fBandLeft write SetfBandLeft;
     property BandTimeLineWidth:byte read fTimeLine write SetfTimeLine;
     property BandTimeLineExtra:byte read fTimeLineExtra write SetfTimeLineExtra;
     property BandTimeLineTransparent:boolean read fTimeTrans write fTimeTrans;

     property CountRows:word read fYCnt write SetfYCnt ;  //
     property CountColums:word read fXcnt write SetfXCnt ;  //
     property CountItems:word read fEcnt write SetfECnt;
     property TextCenter:boolean read fCenter write fCenter;
     property TextCount:byte read fTCnt write SetfTcnt;
     property TextWidth:word read fTX write SetfTX ;      //>16
     property TextColor:Tcolor read fTextColor write fTextColor;
     property TextTransparent:Boolean read fTrans write fTrans;
     property TextFocusColor:Tcolor read fFocusColor write fFocusColor;
     property TextFocusFontColor:Tcolor read fFocusFontColor write fFocusFontColor;
     property ImageShow:boolean read fImageShow write SetfImageShow;
     property ImageList:TImageList read fImageList write fImageList;
     property GridDouble:boolean read fDouble write SetfDouble;
     property GridYBorder:byte read fLM write setfLM;
     property GridXBorder:byte read fCM write SetfCM;
     property Wallpaper: TPicture read FWallpaper write SetWallpaper;
     property ActFocus:TELabel read fActFocus write fActFocus;
     property ReadOnlyGridWidth :word read fx;
     property ReadOnlyGridHeight:word read fy;

     property OnSize:TnotifyEvent read fNewSize write fNewSize;
     property OnGetData:TGridPanelGetData read fGetData write fGetData;
     property OnGetTopData:TGridPanelGetBandData read fGetTop write fGetTop;
     property OnGetLeftData:TGridPanelGetBandData read fGetLeft write fGetLeft;
     property OnAfterDrawText:TGridPanelDrawText read fDrawText write fDrawText;
     property OnAfterRefresh:TnotifyEvent read fAfterRefresh write fAfterRefresh;
     property OnBeforeRefresh:TnotifyEvent read fBeforeRefresh write fBeforeRefresh;
     property OnLabelClick:TnotifyEvent read fLabelClick write fLabelClick;
     property OnLabelDblClick:TnotifyEvent read fLabelDblClick write fLabelDblClick;
     property OnImageClick:TnotifyEvent read fImageClick write fImageClick;
     property OnImageDblClick:TnotifyEvent read fImageDblClick write fImageDblClick;
     property OnFocusChange:TnotifyEvent read fFocusChange write fFocusChange;
     property OnLeftLabelDblClick:TnotifyEvent read fLeftLabelDblClick write fLeftLabelDblClick;

     procedure DataRefresh;
     procedure DrawGrid(x,y:word);
     procedure LabelFocus(pLabel:TELabel; pMove:boolean=false);
     procedure LabelSetFocus(x,y,index:integer);
     function  FocusByTagId(pDbId:integer):boolean;
     function  FindLabel(pDbId:integer):TElabel;
     procedure ClearAll;
     procedure TextFormat(var mtext:String);
     procedure ToMove(x,y,ind:integer; pNext:boolean=true);
     procedure InsertItem(var G:TgridItem);
     procedure DeleteItem(DBID:integer);
    { Published declarations }
  end;


{procedure Register;}

implementation
const cTextBevel=3;

Destructor TGridPanel.Destroy;
begin
  if fIMG1<>nil then fIMG1.free;
  fIMG1:=nil;
  FWallpaper.OnChange := nil;
  FWallpaper.Free;
  FWallpaper := nil;
  inherited Destroy;
end;

Constructor TGridPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fActFocus:=nil;
  fFocusColor:=clBlue;
  fFocusFontColor:=clWhite;
  FWallpaper := TPicture.Create;
  FWallpaper.OnChange := WallpaperChanged;
  ftextColor:=clWindow;
  fCenter:=true;
  fTcnt:=1;
  fTx:=50;
  fECnt:=1;
  fXCnt:=10;
  fImageShow:=true;
  fYCnt:=5;
  fLM:=0;
  fCM:=1;
  Caption:='';
  NewSize;
end;

procedure TGridPanel.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = fImageList) then fImageList:=nil;
end;

function TGridPanel.FocusByTagId(pDbId:integer):boolean;
var i:integer;
begin
  result:=false;
  for i:=Self.Owner.ComponentCount-1 downto 0 do begin
    if Pos(Self.Name+'_ZL_',Components[i].Name)=1 then begin
      if Components[i].Tag=pDbId then begin
        result:=true;
        LabelFocus(TElabel(Components[i]),true);
        exit;
      end;
    end;
  end;
end;

function TGridPanel.FindLabel(pDbId:integer):TElabel;
var i:integer;
begin
  result:=nil;
  for i:=Self.owner.ComponentCount-1 downto 0 do begin
    if Pos(Self.Name+'_ZL_',Self.Owner.Components[i].Name)=1 then begin
      if Self.Owner.Components[i].Tag=pDbId then begin
        result:=TElabel(Self.Owner.Components[i]);
        exit;
      end;
    end;
  end;
end;

procedure TGridPanel.ClearAll;
var i:integer;
begin
  for i:=Self.Owner.ComponentCount-1 downto 0 do begin
    if Pos(Self.Name+'_Z',Components[i].Name)=1 then Components[i].Free;
  end;
end;

procedure TGridPanel.WallpaperChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TGridPanel.SetWallpaper(Value: TPicture);
begin
  FWallpaper.Assign(Value);
end;

procedure TGridPanel.NewSize;
begin
  fBevelSize := BorderWidth;
  if BevelOuter <> bvNone then Inc(fBevelSize, BevelWidth);
  if BevelInner <> bvNone then Inc(fBevelSize, BevelWidth);
  fx:=fEcnt*(ftx+2+fTimeLine)+fCM;
  Width:=fTimeLine+fTimeLineExtra+fBandLeft+3+fx*fXcnt+2*fBevelSize;
  if fImageShow then begin
    if fDouble then fy:=fLM+3+17+18+fTcnt*2*(Abs(Font.Height)+3) else fy:=fLM+3+17+fTcnt*(Abs(Font.Height)+3);
  end else begin
    if fDouble then fy:=fLM+6+fTcnt*2*(Abs(Font.Height)+3) else fy:=fLM+5+fTcnt*(Abs(Font.Height)+3);
  end;
  Height:=fBandTop+1+fy*fYCnt+2*fBevelSize;
  if (self.Parent is TScrollPanel) and (TscrollPanel(Self.Parent).ScrollVert<>nil) then begin
    with TscrollPanel(Self.Parent).ScrollVert do begin
      LargeChange:=fy;
      PageSize:=Self.Parent.height-1;
    end;
  end;
  if (self.Parent is TScrollPanel) and (TscrollPanel(Self.Parent).ScrollHorz<>nil) then begin
    with TscrollPanel(Self.Parent).ScrollHorz do begin
      LargeChange:=ftx+2+fCM;
      PageSize:=Self.Parent.width-1;
      TscrollPanel(Self.Parent).SetScrolls;
    end;
  end;
  if Assigned(fNewSize) then fNewSize(self);
end;

procedure TGridPanel.SetfBandTop(pA:word);
begin
 fBandTop:=pA;
 NewSize;
end;

procedure TGridPanel.SetfBandLeft(pA:word);
begin
 fBandLeft:=pA;
 NewSize;
end;

procedure TGridPanel.SetfTimeLine(pA:byte);
begin
 fTimeLine:=pA;
 NewSize;
end;

procedure TGridPanel.SetfTimeLineExtra(pA:byte);
begin
 fTimeLineExtra:=pA;
 NewSize;
end;

procedure TGridPanel.SetfImageShow(pB:boolean);
begin
  fImageShow:=pB;
  NewSize;
end;

procedure TGridPanel.SetfDouble(pB:boolean);
begin
  fDouble:=pB;
  NewSize;
end;

procedure TGridPanel.SetfTCnt(pCnt:byte);
begin
  if pCnt<1 then pCnt:=1;
  if pCnt>4 then pCnt:=4;
  fTCnt:=pCnt;
  NewSize;
end;

procedure TGridPanel.SetfLM(pCnt:byte);
begin
  if pCnt>15 then pCnt:=15;
  fLM:=pCnt;
  NewSize;
end;

procedure TGridPanel.SetfCM(pCnt:byte);
begin
  if pCnt>50 then pCnt:=50;
  fCM:=pCnt;
  NewSize;
end;

procedure TGridPanel.SetfXCnt(pCnt:word);
begin
  if pCnt<1 then pCnt:=1;
  fXCnt:=pCnt;
  NewSize;
end;

procedure TGridPanel.SetfYCnt(pCnt:word);
begin
  if pCnt<1 then pCnt:=1;
  fYCnt:=pCnt;
  NewSize;
end;

procedure TGridPanel.SetfECnt(pCnt:word);
begin
  if pCnt<1 then pCnt:=1;
  fECnt:=pCnt;
  NewSize;
end;

procedure TGridPanel.SetfTX(pWidth:word);
begin
  if pWidth<20 then pWidth:=20;
  fTx:=pWidth;
  NewSize;
end;

procedure TGridPanel.DrawLeft;
var i:word;
begin
  for i:=0 to fYcnt-1 do begin
    DrawLeftGrid(i);
  end;
end;

procedure TGridPanel.DrawTop;
var i:word;
begin
  for i:=0 to fXcnt-1 do begin
    DrawTopGrid(i);
  end;
end;

procedure TGridPanel.DataRefresh;
var x,y:word;
begin
  Screen.Cursor:=crHourGlass;
  try
    if fActFocus<>nil then fActFocus.Font.Color:=Self.Font.Color;
    fActFocus:=nil;
    if Assigned(fBeforeRefresh) then fBeforeRefresh(self);
    NewSize;
    if fBandTop>0 then DrawTop;
    if fBandleft>0 then DrawLeft;
    for y:=0 to fYcnt-1 do begin
      for x:=0 to fXcnt-1 do begin
        DrawGrid(x,y);
      end;
    end;
    if fActFocus<>nil then begin
      fActFocus.Color:=fFocusColor;
      fActFocus.Font.Color:=fFocusFontColor;
      fActFocus.Transparent:=false;
      fActFocus.height:=fTcnt*(Abs(Font.height)+3);
    end;
  finally
    Screen.Cursor:=crDefault;
    if Assigned(fAfterRefresh) then fAfterRefresh(self);
  end;
end;

procedure TGridPanel.LabelFocus(pLabel:TELabel;pMove:boolean=false);
var
  mPoint:Tpoint;
  i:integer;
begin
  if pLabel.visible=false then exit;
  if fActFocus<>nil then begin
    if (pMove=false) and (fActFocus=pLabel) then begin
      pLabel.Color:=fFocusColor;
      pLabel.Font.Color:=fFocusFontColor;
      pLabel.Transparent:=false;
      pLabel.height:=fTcnt*(Abs(Font.height)+3);
      pLabel.AutoSize:=false;
      pLabel.Show;
    end;
    fActFocus.Color:=fActColor; fActFocus.Font.Color:=fActFontColor;
    fActFocus.height:=fTcnt*(Abs(Font.height)+3);
    fActFocus.transparent:=FTrans;
    fActFocus.AutoSize:=false;
    fActFocus.Show;
  end;
  Application.processmessages;
  fActFocus:=pLabel;
  fActFocus.AutoSize:=false;
  if pLabel<>nil then begin
    fActFontColor:=pLabel.Font.Color;
    fActColor:=pLabel.Color;
    pLabel.Color:=fFocusColor;
    pLabel.Font.Color:=fFocusFontColor;
    pLabel.Transparent:=false;
    pLabel.height:=fTcnt*(Abs(Font.height)+3);
    pLabel.AutoSize:=false;
    if pMove then begin
      if not (self.Parent is TScrollBox) then
        ShowMessage(Self.name+ '- error focused'+#13#10+'My parent must by a ScrollPanel or ScrollBox')
      else begin
        if (self.Parent is TScrollPanel) and (TscrollPanel(Self.Parent).ScrollVert<>nil) then
        TscrollPanel(Self.Parent).ScrollVert.Position:=0 else TScrollBox(Self.Parent).vertScrollBar.Position:=0;

        if (self.Parent is TScrollPanel) and (TscrollPanel(Self.Parent).ScrollHorz<>nil) then
        TscrollPanel(Self.Parent).ScrollHorz.Position:=0 else TScrollBox(Self.Parent).horzScrollBar.Position:=0;

        mPoint:=TControl(Self.Parent).ScreenToClient(pLabel.ClientOrigin);
        i:=mPoint.Y-TControl(Self.parent).Clientheight+pLabel.Height;
        if i<0 then i:=0;
        if (self.Parent is TScrollPanel) and (TscrollPanel(Self.Parent).ScrollVert<>nil) then
          TscrollPanel(Self.Parent).ScrollVert.Position:=i
        else TScrollBox(Self.parent).vertScrollBar.Position:=i;

        i:=mPoint.X-TControl(Self.parent).Clientwidth+pLabel.Width;
        if i<0 then i:=0;
        if (self.Parent is TScrollPanel) and (TscrollPanel(Self.Parent).ScrollHorz<>nil) then
          TscrollPanel(Self.Parent).ScrollHorz.Position:=i
        else TScrollBox(Self.parent).HorzScrollBar.Position:=i;
      end;
    end;
  end;
  Application.processmessages;
  if Assigned(fFocusChange) then fFocusChange(pLabel);
end;

procedure TGridPanel.LabelSetFocus(x,y,index:integer);
var mC:TComponent;
begin
  mC:=Self.Owner.findComponent(Self.Name+'_ZL_'+IntToStr(x)+'_'+IntToStr(y)+'_'+IntToStr(Index));
  if mC<>nil then LabelFocus(TElabel(mC),true);
end;

procedure TGridPanel.Paint;
var
  XCnt, YCnt, X, Y: Integer;
  BevelSize, SaveIndex: Integer;
  mRect: TRect;
begin
  inherited Paint;
  if (FWallpaper.Graphic<>nil) and (FWallpaper.Width>0) and (FWallpaper.Height>0) then
  begin
    mRect := Self.ClientRect;
    BevelSize := Self.BorderWidth;
    if BevelOuter <> bvNone then Inc(BevelSize, BevelWidth);
    if BevelInner <> bvNone then Inc(BevelSize, BevelWidth);
    InflateRect(mRect, -BevelSize, -BevelSize);
    SaveIndex := SaveDC(Self.Canvas.Handle);
    try
        IntersectClipRect(Self.Canvas.Handle, mRect.Left, mRect.Top, mRect.Right -mRect.Left +
          BevelSize, mRect.Bottom - mRect.Top + BevelSize);
        XCnt :=(Self.ClientWidth - 2 * BevelSize) div FWallpaper.Width;
        YCnt :=(Self.ClientHeight - 2 * BevelSize) div FWallpaper.Height;
        for X := 0 to XCnt do
          for Y := 0 to YCnt do
            self.Canvas.Draw(mRect.Left + X * FWallpaper.Width,
              mRect.Top + Y * FWallpaper.Height, FWallpaper.Graphic);
    finally
      RestoreDC(self.Canvas.Handle, SaveIndex);
    end;
  end;
end;

procedure TGridPanel.LabelClick(Sender: TObject);
begin
  LabelFocus(TElabel(sender));
  if Assigned(fLabelClick) then fLabelClick(sender);
end;

procedure TGridPanel.ImageClick(Sender: TObject);
var mS:String;
    mC:Tcomponent;
begin
  mS:=Tcomponent(Sender).Name;
  mS[Length(Self.Name)+3]:='L';
  mC:=Self.Owner.FindComponent(mS);
  if mC<>nil then LabelFocus(TElabel(mC));
  if Assigned(fImageClick) then fImageClick(sender);
end;

procedure TGridPanel.LabelDblClick(Sender: TObject);
begin
  LabelFocus(TElabel(sender));
  PostMessage(Handle,UM_labelDblClick,0,0);
  //toto som musel urobit preto lebo ak Label sa odstranila (Free) tak nastala nejaka cudna chyba
end;

procedure TGridPanel.LeftLabelDblClick(Sender: TObject);
begin
  if Assigned(fLeftLabelDblClick) then fleftLabelDblClick(Sender);
end;

procedure TGridPanel.Proc_labelDblClick(var Msg: TMessage);
begin
  //toto som musel urobit preto lebo ak Label sa odstranila (Free) tak nastala nejaka cudna chyba
  if fActFocus<>nil then
    if Assigned(fLabelDblClick) then fLabelDblClick(TELabel(fActFocus));
end;

procedure TGridPanel.ImageDblClick(Sender: TObject);
var mS:String;
    mC:Tcomponent;
begin
  mS:=Tcomponent(Sender).Name;
  mS[Length(Self.Name)+3]:='L';
  mC:=Self.Owner.FindComponent(mS);
  if mC<>nil then LabelFocus(TElabel(mC));
  if Assigned(fImageDblClick) then fImageDblClick(sender);
end;

procedure TGridPanel.TextFormat(var mtext:String);
var i,j,k:integer;
    mS,mT,mR,s:string;
    mB:boolean;
    c:Tcolor;
begin
  i:=0;
  if fIMG1=nil then begin
    fIMG1:=Timage.Create(self.owner);
    fIMG1.Parent:=self;
    fIMG1.Name:=Self.Name+'XXXIMG1';
    fIMG1.Visible:=false;
    fIMG1.transparent:=true;
  end;
  with fIMG1.canvas do begin
    Font:=Self.Font;mR:='';
    mS:=trim(mText);
    if mS<>'' then
    i:=Pos(#13#10,mS);j:=0;
    while i>0 do begin
      inc(j);
      if i>1 then begin
        mT:=copy(mS,1,i-1);
        if Length(mT)>35 then mT:=copy(mT,1,35);
        while TextWidth(mT)>ftx-cTextBevel do delete(mT,Length(mT),1);
      end else mT:='';
      mR:=mR+mT+#13#10;
      delete(mS,1,i+1);
      i:=Pos(#13#10,mS);
    end;
    if mS<>'' then begin
      while TextWidth(mS)>ftx-cTextBevel do delete(mS,Length(mS),1);
      mR:=mR+mS;
    end;
  end;
  mText:=mR;
end;

procedure TGridPanel.DrawGrid(x,y:word);
var mOk:boolean;
    mText,mHint:string;
    mImageIndex:integer;
    mImage,mTime,mTimeEx:TImage;
    mLabel:TELabel;
    mCnt,mItemNo:word;
    mbevel:Tbevel;
    mC:TComponent;
    mDBID,mId:integer;
    mTimeSize:integer;
    mColor:Tcolor;

procedure TextCat;
var i,j,k:integer;
    mS,mT,mR,s:string;
    mB:boolean;
    c:Tcolor;
begin
  i:=0;
  if fIMG1=nil then begin
    fIMG1:=Timage.Create(self.owner);
    fIMG1.Parent:=self;
    fIMG1.Name:=Self.Name+'XXXIMG1';
    fIMG1.Visible:=false;
    fIMG1.transparent:=true;
  end;
  with fIMG1.canvas do begin
    Font:=Self.Font;mR:='';
    mS:=trim(mText);
    if mS<>'' then
    i:=Pos(#13#10,mS);j:=0;
    while i>0 do begin
      inc(j);
      if i>1 then begin
        mT:=copy(mS,1,i-1);
        if Length(mT)>35 then mT:=copy(mT,1,35);
        while TextWidth(mT)>ftx-cTextBevel do delete(mT,Length(mT),1);
      end else mT:='';
      if (j=1) and (mItemNo>=mCnt-1) and assigned(fGetData) then begin
         mB:=true;k:=0;
         fGetData(Self,s,s,k,k,k,mB,c,x,y,mItemNo,true);
         if mB then begin
           while TextWidth(mT+'>')>ftx-cTextBevel do delete(mT,Length(mT),1);
           mT:=mT+'>';
         end;
      end;
      mR:=mR+mT+#13#10;
      delete(mS,1,i+1);
      i:=Pos(#13#10,mS);
    end;
    if mS<>'' then begin
      while TextWidth(mS)>ftx-cTextBevel do delete(mS,Length(mS),1);
      mR:=mR+mS;
    end;
  end;
  mText:=mR;
end;

procedure MyPos;
var mI:word;
begin
  if fDouble then mI:=(mItemNo+1) div 2 else mI:=mItemNo;
  mLabel.left:=fTimeLineExtra+fTimeLine+fBandLeft+fTimeLine+fBevelSize+2+x*fx+(mI-1)*(ftx+ftimeLine+2);
  mLabel.Height:=fTcnt*(Abs(Font.height)+3);
  mLabel.Width:=ftx-cTextBevel;
  mLabel.Color:=ftextColor;
  if fCenter then mLabel.Alignment:=taCenter else mLabel.Alignment:=taLeftJustify;
  mLabel.Transparent:=fTrans;
  if fimageShow then begin
    mImage.left:=FtimeLineExtra+fTimeLine+fBandLeft+fTimeLine+fBevelSize+2+x*fx+(mI-1)*(ftx+fTimeLine+2)+((ftx+2) div 2)-8;
    if fDouble and (mItemNo mod 2=0) then begin
      mLabel.Top:=fbandTop+fBevelSize+1+y*fy+17+18+fTcnt*(Abs(font.height)+3);
      mImage.top:=fBandTop+fBevelSize+1+y*fy+17+2+fTcnt*(Abs(font.height)+3);
    end else begin
      mLabel.Top:=fBandTop+fBevelSize+1+y*fy+17;
      mImage.Top:=fBandTop+fBevelSize+1+y*fy+1;
    end;
  end else begin
    if fdouble and (mItemNo mod 2=0) then begin
      mLabel.top:=fBandTop+fBevelSize+1+y*fy+3+fTcnt*(Abs(font.height)+3);
    end else mLabel.Top:=fBandTop+fBevelSize+1+y*fy+2;
  end;
end;

begin
  mOk:=true; mItemNo:=0;
  mCnt:=fEcnt;  if fDouble then mCnt:=2*mCnt; mHint:='';
  while mOk and (mItemNo<mCnt) do begin
    inc(mItemNo);mImageIndex:=-1; mDBID:=0; mText:='';mTimeSize:=0; mColor:=Font.Color;
    if assigned(fGetData) then fGetData(Self,mText,mHint,mImageIndex,mDBID,mTimeSize,mOk,mColor,x,y,mItemNo) else mOk:=false;
    if mOK then begin
      TextCat;
      mC:=Self.Owner.findComponent(Self.Name+'_ZL_'+IntToStr(x)+'_'+IntToStr(y)+'_'+IntToStr(mItemNo));
      if mC<>nil then mLabel:=TELabel(mC)
      else begin
        mLabel:=TELabel.Create(self.Owner);
        mLabel.Parent:=Self;
        mLabel.Name:=Self.Name+'_ZL_'+IntToStr(x)+'_'+IntToStr(y)+'_'+IntToStr(mItemNo);
      end;
      mLabel.MyX:=x;
      mLabel.MyY:=y;
      mLabel.MyIndex:=mItemNo;
      mLabel.font.Color:=mColor;
      mLabel.caption:=mText;
      mLabel.Tag:=mDBID;
      mLabel.OnClick:=LabelClick;
      mLabel.OnDblClick:=LabelDblClick;
      mLabel.ShowHint:=true;
      mLabel.Hint:=mHint;
      if mTimeSize<0 then mTimeSize:=0;
      mLabel.TimeSize:=mTimeSize;
      if fImageShow then begin
        mC:=Self.Owner.findComponent(Self.Name+'_ZI_'+IntToStr(x)+'_'+IntToStr(y)+'_'+IntToStr(mItemNo));
        if mC<>nil then mImage:=Timage(mC)
        else begin
          mImage:=TImage.Create(Self.Owner);
          mImage.Parent:=self;
          mImage.Name:=self.Name+'_ZI_'+IntToStr(x)+'_'+IntToStr(y)+'_'+IntToStr(mItemNo);
        end;
        mImage.Transparent:=TRUE;
        mImage.AutoSize:=false;
        mImage.Width:=17;
        mImage.Height:=17;
        mImage.Tag:=mDBID;
        mImage.OnClick:=ImageClick;
        mImage.OnDblClick:=ImageDblClick;
        if (mImageIndex=-1) or (fImageList=nil) then begin
          mImage.Canvas.Pen.Style :=psSolid;
          mImage.Canvas.Pen.Color:=clAqua;
          mImage.Canvas.Brush.Color:=clOlive;
          mImage.Canvas.Rectangle(0,0,16,16);
        end else begin
          if fImageList<>nil then begin
            fImageList.getIcon(mImageIndex,mImage.Picture.Icon);
          end;
          mImage.Stretch:=true;
        end;

      end;
      MyPos;
      if Assigned(fDrawText) then fDrawText(self,mLabel,x,y,mItemNo);
    end;
  end;
  mC:=Self.Owner.findComponent(self.Name+'_ZB_'+IntToStr(x)+'_'+IntToStr(y));
  if mC<>nil then mBevel:=Tbevel(mC)
  else begin
    mbevel:=Tbevel.Create(self.owner);
    mbevel.Parent:=self;
    mBevel.Name:=self.Name+'_ZB_'+IntToStr(x)+'_'+IntToStr(y);
  end;
  mbevel.Shape:= bsFrame;
  mbevel.Style:=bsLowered;
  mbevel.width:=fx+2-fCM;
  mbevel.Height:=fy+1-fLM;
  mBevel.top:=fBandTop+fBevelSize+y*fy;
  mBevel.Left:=fTimeLine+fTimeLineExtra+fBandleft+fBevelSize+x*fx;
  if x=0 then begin
    mBevel.Left:=mBevel.left-fTimeLine;
    mBevel.Width:=mBevel.Width+fTimeLine;
  end;
  mBevel.SendToBack;
  if fTimeLine>0 then begin
    mC:=Self.Owner.findComponent(self.Name+'_ZT_'+IntToStr(x));
    if mC<>nil then mTime:=TImage(mC)
    else begin
      mTime:=TImage.Create(self.owner);
      mTime.Parent:=self;
      mTime.Name:=self.Name+'_ZT_'+IntToStr(x);
    end;
    mTime.width:=fTimeLine;
    mTime.Height:=Height-fBandTop-fBevelSize-2;
    mTime.top:=fBandTop+fBevelSize+1;
    mTime.Left:=fTimeLine+fTimeLineExtra+fBandleft+fBevelSize+x*fx+1;
    if x=0 then begin
      mTime.Left:=mTime.left-fTimeLine;
      mTime.Width:=mTime.Width+fTimeLine;
    end;
    mTime.Transparent:=fTimeTrans;
    mTime.canvas.Pixels[0,0]:=clYellow;
  end;
  if (fTimeLineExtra>0) and (x=0) then begin
    mC:=Self.Owner.findComponent(self.Name+'_ZTE_'+IntToStr(x));
    if mC<>nil then mTimeEx:=TImage(mC)
    else begin
      mTimeEx:=TImage.Create(self.owner);
      mTimeEx.Parent:=self;
      mTimeEx.Name:=self.Name+'_ZTE_'+IntToStr(x);
    end;
    mTimeEx.width:=fTimeLineExtra;
    mTimeEx.Height:=Height-fBandTop-fBevelSize-2;
    mTimeEx.top:=fBandTop+fBevelSize+1;
    mTimeEx.Left:=fBandleft+fBevelSize+x*fx;
    mTimeEx.canvas.Pixels[0,0]:=clYellow;
    mTimeEx.Transparent:=fTimeTrans;
    mTimeEx.canvas.Pixels[0,0]:=clYellow;
  end;
  mId:=mItemNo;
  if mOk then inc(mId);
  mC:=Self.Owner.findComponent(Self.Name+'_ZL_'+IntToStr(x)+'_'+IntToStr(y)+'_'+IntToStr(mId));
  while mC<>nil do begin
    mC.free;
    inc(mId);
    mC:=Self.Owner.findComponent(Self.Name+'_ZL_'+IntToStr(x)+'_'+IntToStr(y)+'_'+IntToStr(mId));
  end;
  mId:=mItemNo;
  if mOK then inc(mId);
  mC:=Self.Owner.findComponent(Self.Name+'_ZI_'+IntToStr(x)+'_'+IntToStr(y)+'_'+IntToStr(mId));
  while mC<>nil do begin
    mC.free;
    inc(mId);
    mC:=Self.Owner.findComponent(Self.Name+'_ZI_'+IntToStr(x)+'_'+IntToStr(y)+'_'+IntToStr(mId));
  end;
  if y>10 then begin
    Screen.Cursor:=crDefault;
    Application.ProcessMessages;
  end;
end;

procedure TGridPanel.DrawTopGrid(grid:word);
var
  mC:Tcomponent;
  mText1,mText2,mHint:string;
  mL1,mL2:Tlabel;
  mBevel:Tbevel;
  mColor1,mColor2:Tcolor;
begin
  mText1:='';mText2:=''; mHint:=''; mColor2:=fTextColor; mColor1:=TScrollbox(Self.Parent).Font.Color;
  if assigned(fGetTop) then fGetTop(Self,mText1,mText2,mHint,mColor1,mColor2,grid);
  mC:=Self.Owner.findComponent(Self.Name+'_ZLT1_'+IntToStr(grid));
  if mC<>nil then mL1:=TLabel(mC)
  else begin
    mL1:=TLabel.Create(self.Owner);
    mL1.Parent:=Self;
    mL1.Name:=Self.Name+'_ZLT1_'+IntToStr(grid);
    mL1.font:=TScrollbox(Self.Parent).Font;
  end;
  mL1.Transparent:=true;
  mL1.AutoSize:=true;
  mL1.AutoSize:=false;
  mL1.Width:=fx-6;
  mL1.Color:=mColor1;
  mL1.caption:=mText1;
  mL1.Alignment:=taCenter;
  mL1.top:=13;
  mL1.left:=fTimeLine+fTimeLineExtra+fBandLeft+fBevelSize+2+grid*fx+2;
  if grid=0 then begin
    mL1.Left:=mL1.Left-fTimeLine;
    mL1.width:=mL1.Width+fTimeLine;
  end;
  mL1.Hint:=mHint;
  mL1.ShowHint:=true;
  //mL2
  mC:=Self.Owner.findComponent(Self.Name+'_ZLT2_'+IntToStr(grid));
  if mC<>nil then mL2:=TLabel(mC)
  else begin
    mL2:=TLabel.Create(self.Owner);
    mL2.Parent:=Self;
    mL2.Name:=Self.Name+'_ZLT2_'+IntToStr(grid);
  end;
  mL2.Transparent:=true;
  mL2.AutoSize:=true;
  mL2.AutoSize:=false;
  mL2.Width:=mL1.width;
  mL2.Font.Name:='MS Serif';
  mL2.Font.Size:=7;
  mL2.font.Style:=[];
  mL2.Font.Color:=mColor2;
  mL2.Color:=ftextColor;
  mL2.Alignment:=taRightJustify;
  mL2.caption:=mText2;
  mL2.top:=3; mL2.left:=mL1.left;
  mC:=Self.Owner.findComponent(self.Name+'_ZBT_'+IntToStr(grid));
  if mC<>nil then mBevel:=Tbevel(mC)
  else begin
    mbevel:=Tbevel.Create(self.owner);
    mbevel.Parent:=self;
    mBevel.Name:=self.Name+'_ZBT_'+IntToStr(grid);
  end;
  mbevel.Shape:= bsFrame;
  mbevel.Style:=bsLowered;
  mbevel.width:=fx+2-fCM; //-fTimeLine;
  mbevel.Height:=fBandTop;
  mBevel.Top:=0;
  mBevel.Left:=fTimeLine+fTimeLineExtra+fBandleft+fBevelSize+grid*fx; //+TimeLine
  if grid=0 then begin
    mBevel.Left:=mbevel.left-fTimeLine;
    mBevel.width:=mBevel.Width+fTimeLine;
  end;
  mBevel.SendToBack;
  Invalidate;
  Application.processmessages;
end;

procedure TGridPanel.DrawLeftGrid(grid:word);
var
  mC:Tcomponent;
  mText1,mText2,mHint:string;
  mL1,mL2:Tlabel;
  mBevel:Tbevel;
  mColor1,mColor2:Tcolor;
begin
  mText1:='';mText2:=''; mHint:='';mColor1:=TScrollbox(Self.Parent).Font.Color; mColor2:=ftextColor;
  if assigned(fGetLeft) then fGetLeft(Self,mText1,mText2,mHint,mColor1,mColor2,grid);
  mC:=Self.Owner.findComponent(Self.Name+'_ZLL1_'+IntToStr(grid));
  if mC<>nil then mL1:=TLabel(mC)
  else begin
    mL1:=TLabel.Create(self.Owner);
    mL1.Parent:=Self;
    mL1.Name:=Self.Name+'_ZLL1_'+IntToStr(grid);
    mL1.font:=TScrollbox(Self.Parent).Font;
    mL1.OnDblClick:=LeftLabelDblClick;
  end;
  mL1.Transparent:=true;
  mL1.AutoSize:=true;
  mL1.AutoSize:=false;
  mL1.Width:=fBandLeft-5;
  mL1.Font.Color:=mColor1;
  mL1.caption:=mText1;
  mL1.Alignment:=taCenter;
  mL1.top:=12+fBandTop+fBevelSize+1+grid*fy;
  mL1.left:=2;
  mL1.Hint:=mHint;
  mL1.ShowHint:=true;
  //mL2
  mC:=Self.Owner.findComponent(Self.Name+'_ZLL2_'+IntToStr(grid));
  if mC<>nil then mL2:=TLabel(mC)
  else begin
    mL2:=TLabel.Create(self.Owner);
    mL2.Parent:=Self;
    mL2.Name:=Self.Name+'_ZLL2_'+IntToStr(grid);
  end;
  mL2.Font.Name:='MS Serif';
  mL2.Font.Size:=7;
  mL2.font.Style:=[fsUnderline];
  mL2.Font.Color:=mColor2;
  mL2.AutoSize:=true;
  mL2.AutoSize:=false;
  mL2.Width:=mL1.width;
  mL2.Color:=ftextColor;
  mL2.Alignment:=taRightJustify;
  mL2.Transparent:=true;
  mL2.caption:=mText2;
  mL2.top:=mL1.top-10; mL2.left:=mL1.left;
  mC:=Self.Owner.findComponent(self.Name+'_ZBL_'+IntToStr(grid));
  if mC<>nil then mBevel:=Tbevel(mC)
  else begin
    mbevel:=Tbevel.Create(self.owner);
    mbevel.Parent:=self;
    mBevel.Name:=self.Name+'_ZBL_'+IntToStr(grid);
  end;
  mbevel.Shape:= bsTopLine;
  mbevel.Style:=bsLowered;
  mbevel.width:=fBandLeft; //-fTimeLine;
  mbevel.Height:=fy-1-fLM;
  mBevel.Top:=fBandTop+fBevelSize+grid*fy-1;
  mBevel.Left:=0; //+TimeLine
  mBevel.SendToBack;
  Application.processmessages;
end;

procedure TGridPanel.ToMove(x,y,ind:integer; pNext:boolean=true);
var i,mi,mmi:integer;
    mL:TELabel;
    mE:integer;
    mImg:Timage;
    mC:Tcomponent;
    mS,mSS:String;
    mA:array [1..60] of TELabel;
    mB:array [1..60] of TImage;
begin
  mE:=1;mmi:=0;
  for i:=1 to 60 do  mA[i]:=nil;
  if not pNext then mE:=-1;
  for i:=Self.owner.ComponentCount-1 downto 0 do begin
    if (Self.Owner.Components[i] is TELabel) then begin
      mL:=TELabel(Self.Owner.Components[i]);
      if (mL.MyX=x) and (mL.MyY=y) and (mL.MyIndex>=ind) then begin
        if ind+mE>0 then begin
          mL.left:=GetLabelX(x,y,mL.MyIndex+mE);
          mi:=mL.MyIndex;  if mi>mmi then mmi:=mi;
          mA[mi]:=mL;
          mL.MyIndex:=mi+mE;
          mS:=Self.Name+'_ZI_'+IntToStr(x)+'_'+IntToStr(y)+'_'+IntToStr(mi);
          mC:=Self.Owner.FindComponent(mS);
          if mC is TImage  then begin
            mImg:=TImage(mC);
            mB[mi]:=mImg;
            mImg.Left:=GetImageX(x,y,mi+mE);
          end;
        end;
      end;
    end;
  end;
  mS:=Self.Name+'_ZL_'+IntToStr(x)+'_'+IntToStr(y)+'_';
  mSS:=Self.Name+'_ZI_'+IntToStr(x)+'_'+IntToStr(y)+'_';
  if mE>0 then begin
    for i:=mmi downto 1 do begin
      if mA[i]<>nil then  begin
        try mA[i].name:=mS+IntToStr(mA[i].MyIndex); except end;
      end;
      if (mB[i]<>nil) and (mA[i]<>nil) then begin
        try mB[i].name:=mSS+IntToStr(mA[i].MyIndex); except end;
      end;
    end;
  end else begin
    for i:=1 to mmi do begin
      if mA[i]<>nil then  begin
        try mA[i].name:=mS+IntToStr(mA[i].MyIndex); except end;
        try if mB[i]<>nil then mB[i].name:=mSS+IntToStr(mA[i].MyIndex);except end;
      end;
    end;
  end;
end;

function TGridPanel.GetLabelX(x,y,mi:integer):integer;
begin
 result:=fTimeLineExtra+fTimeLine+fBandLeft+fTimeLine+fBevelSize+2+x*fx+(mI-1)*(ftx+ftimeLine+2);
end;

function TGridPanel.GetLabelY(x,y,mi:integer):integer;
begin
 result:=fBandTop+fBevelSize+1+y*fy+17;
end;

function TGridPanel.GetImageX(x,y,mi:integer):integer;
begin
 result:=FtimeLineExtra+fTimeLine+fBandLeft+fTimeLine+fBevelSize+2+x*fx+(mI-1)*(ftx+fTimeLine+2)+((ftx+2) div 2)-8;
end;

function TGridPanel.GetImageY(x,y,mi:integer):integer;
begin
  result:=fBandTop+fBevelSize+1+y*fy+1;
end;

procedure TGridPanel.CreateItem(var G:TGridItem);
var mLabel:TElabel;
    mImage:TImage;
    mS:String;
begin
  mLabel:=TELabel.Create(self.Owner);
  mLabel.Parent:=Self;
  mS:=Self.Name+'_ZL_'+IntToStr(G.px)+'_'+IntToStr(G.py)+'_'+IntToStr(G.pind);
  while Self.Owner.FindComponent(mS)<>nil do begin
    G.pInd:=G.pInd+1;
    mS:=Self.Name+'_ZL_'+IntToStr(G.px)+'_'+IntToStr(G.py)+'_'+IntToStr(G.pind);
  end;
  mLabel.Name:=Self.Name+'_ZL_'+IntToStr(G.px)+'_'+IntToStr(G.py)+'_'+IntToStr(G.pind);
  mLabel.Hint:=G.pHint;
  mLabel.Caption:=G.pText;
  mLabel.MyX:=G.px;
  mLabel.MyY:=G.py;
  mLabel.MyIndex:=G.pind;
  mLabel.font.Color:=G.pColor;
  mLabel.Tag:=G.pNewID;
  G.pLabel:=mLabel;
  mLabel.OnClick:=LabelClick;
  mLabel.OnDblClick:=LabelDblClick;
  mLabel.ShowHint:=true;
  if G.pTimeSize<0 then G.pTimeSize:=0;
  mLabel.AutoSize:=false;
  mLabel.TimeSize:=G.pTimeSize;
  mLabel.left:=getLabelX(G.px,G.py,g.pInd);
  mLabel.Top:=getLabelY(G.px,G.py,g.pInd);
  mLabel.Height:=fTcnt*(Abs(Font.height)+3);
  mLabel.Width:=ftx-cTextBevel;
  mLabel.Color:=ftextColor;
  if fCenter then mLabel.Alignment:=taCenter else mLabel.Alignment:=taLeftJustify;
  mLabel.Transparent:=fTrans;
  mLabel.AutoSize:=false;
  if fImageShow then begin
    mImage:=TImage.Create(Self.Owner);
    mImage.Parent:=self;
    mImage.Name:=self.Name+'_ZI_'+IntToStr(G.px)+'_'+IntToStr(G.py)+'_'+IntToStr(G.pind);
    mImage.Transparent:=TRUE;
    mImage.AutoSize:=false;
    mImage.Width:=17;
    mImage.Height:=17;
    mImage.left:=getImageX(G.px,G.py,g.pInd);
    mImage.Top:=getImageY(G.px,G.py,g.pInd);
    mImage.Tag:=G.pNewID;
    G.pImage:=mImage;
    mImage.OnClick:=ImageClick;
    mImage.OnDblClick:=ImageDblClick;
    if (G.pImageIndex=-1) or (fImageList=nil) then begin
      mImage.Canvas.Pen.Style :=psSolid;
      mImage.Canvas.Pen.Color:=clAqua;
      mImage.Canvas.Brush.Color:=clOlive;
      mImage.Canvas.Rectangle(0,0,16,16);
    end else begin
      if fImageList<>nil then begin
        fImageList.getIcon(G.pImageIndex,mImage.Picture.Icon);
      end;
      mImage.Stretch:=true;
    end;
  end;
end;

procedure TGridPanel.InsertItem(var G:TGridItem);
var mLabel:TELabel;
    mC:TComponent;
    mI:integer;
begin
  if (G.pNewID>0) then begin
    mC:=nil;
    if G.pID>0 then mC:=FindLabel(G.pID); //odkial
    if mC<>nil then begin
      mLabel:=TElabel(mC);
      ToMove(mLabel.MyX,mLabel.MyY,mLabel.MyIndex,true);
    end;
    CreateItem(G);
  end;
end;

procedure TGridPanel.DeleteItem(DBID:integer);
var i,x,y,ind:integer;
    mS:string;
    mC:Tcomponent;
    mL:TElabel;
begin
  mC:=FindLabel(DBID);
  if mC<>nil then begin
    mL:=TElabel(mC);
    mS:=mL.Name;
    i:=Pos('_ZL_',mS);
    mS[i+2]:='I';
    Ind:=mL.MyIndex;
    x:=mL.MyX;
    y:=mL.MyY;
    mL.Free;
    mC:=Self.Owner.FindComponent(mS);
    if mC<>nil then TImage(mC).free;
  end;
  ToMove(x,y,ind,false);
end;
(*
procedure Register;
begin
  RegisterComponents('Samples', [TGridPanel]);
end;
*)


end.
