{*************************************************************}
{            SimpleTCP components for Delphi and C++ Builder  }
{ Version:   1.02b                                            }
{ E-Mail:    info@utilmind.com                                }
{ WWW:       http://www.utilmind.com                          }
{ Created:   July 8, 2000                                     }
{ Modified:  July 11, 2000                                    }
{ Legal:     Copyright (c) 2000, UtilMind Solutions           }
{*************************************************************}
{ SimpleTCP is pack of two components (TSimpleTCPServer and   }
{ TSimpleTCPClient) for working with Asynchronous TCP sockets.}
{*************************************************************}
{ Please see demo program for more information.               }
{*************************************************************}
{                     IMPORTANT NOTE:                         }
{ This software is provided 'as-is', without any express or   }
{ implied warranty. In no event will the author be held       }
{ liable for any damages arising from the use of this         }
{ software.                                                   }
{ Permission is granted to anyone to use this software for    }
{ any purpose, including commercial applications, and to      }
{ alter it and redistribute it freely, subject to the         }
{ following restrictions:                                     }
{ 1. The origin of this software must not be misrepresented,  }
{    you must not claim that you wrote the original software. }
{    If you use this software in a product, an acknowledgment }
{    in the product documentation would be appreciated but is }
{    not required.                                            }
{ 2. Altered source versions must be plainly marked as such,  }
{    and must not be misrepresented as being the original     }
{    software.                                                }
{ 3. This notice may not be removed or altered from any       }
{    source distribution.                                     }
{*************************************************************}

unit uLittleServer;

interface

uses
  Windows, Messages, SysUtils, Classes, WinSock;

{$IFDEF VER90}      {Delphi 2.0}
 {$DEFINE DELPHI2}
{$ENDIF}
{$IFDEF VER93}
  {$DEFINE DELPHI2}   {C++Builder 1.0}
{$ENDIF}

const
  WM_TCPASYNCSELECT = WM_USER + $0001;

type
  TSimpleTCPClient = class;

  TSimpleTCPAcceptEvent = procedure(Sender: TObject; Client: TSimpleTCPClient; var Accept: Boolean) of object;
  TSimpleTCPServerEvent = procedure(Sender: TObject; Client: TSimpleTCPClient) of object;
  TSimpleTCPServerIOEvent = procedure(Sender: TObject; Client: TSimpleTCPClient; Buffer: PChar; Length: Integer) of object;
  TSimpleTCPClientIOEvent = procedure(Sender: TObject; Buffer: PChar; BufLength: Integer) of object;
  TSimpleTCPErrorEvent = procedure(Sender: TObject; Socket: TSocket; ErrorCode: Integer; ErrorMsg: String) of object;

  TCustomSimpleSocket = class(TComponent)
  private
    FAllowChangeHostAndPortOnConnection: Boolean;
    FData: Pointer;
    FHost: String;
    FPort: Word;
    FSocket: TSocket;

    FOnError: TSimpleTCPErrorEvent;

    // For private use
    SockAddrIn: TSockAddrIn;
    HostEnt: PHostEnt;
    PProtoEnt: PProtoEnt;
    WSAData: TWSAData;
    WindowHandle: hWnd;

    procedure WndProc(var Msg: TMessage);
    procedure SocketError(Socket: TSocket; ErrorCode: Integer);
    procedure SetSocket(Value: TSocket);
    function SendTo(Socket: TSocket; Buffer: PChar; Length: Integer): Integer; // bytes sent
  protected
    procedure SetHost(Value: String); virtual;
    procedure SetPort(Value: Word); virtual;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;

    property AllowChangeHostAndPortOnConnection: Boolean read FAllowChangeHostAndPortOnConnection write FAllowChangeHostAndPortOnConnection;
    property Data: Pointer read FData write FData;
    property Host: String read FHost write SetHost;
    property Port: Word read FPort write SetPort;
    property Socket: TSocket read FSocket write SetSocket;

    property OnError: TSimpleTCPErrorEvent read FOnError write FOnError;
  end;

  { TSimpleTCPServer }
  TSimpleTCPServer = class(TCustomSimpleSocket)
  private
    FListen: Boolean;
    FLocalHostName: String;
    FLocalIP: String;
    FConnections: TList;

    FOnAccept: TSimpleTCPAcceptEvent;
    FOnClientConnected: TSimpleTCPServerEvent;
    FOnClientDisconnected: TSimpleTCPServerEvent;
    FOnClientRead: TSimpleTCPServerIOEvent;

    procedure SetListen(Value: Boolean);
    function GetLocalHostName: String;
    function GetLocalIP: String;
    procedure SetNoneStr(Value: String);
    procedure WMServerSelect(var Msg: TMessage); message WM_TCPASYNCSELECT;
  protected
    procedure SetPort(Value: Word); override;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;

    function Send(Client: TSimpleTCPClient; Buffer: PChar; Length: Integer): Integer; // bytes sent
    procedure SendToAll(Buffer: PChar; Length: Integer);    

    property Connections: TList read FConnections write FConnections;
  published
    property Listen: Boolean read FListen write SetListen;
    property LocalHostName: String read GetLocalHostName write SetNoneStr;
    property LocalIP: String read GetLocalIP write SetNoneStr;    

    property OnAccept: TSimpleTCPAcceptEvent read FOnAccept write FOnAccept;
    property OnClientConnected: TSimpleTCPServerEvent read FOnClientConnected write FOnClientConnected;
    property OnClientDisconnected: TSimpleTCPServerEvent read FOnClientDisconnected write FOnClientDisconnected;
    property OnClientRead: TSimpleTCPServerIOEvent read FOnClientRead write FOnClientRead;

    property AllowChangeHostAndPortOnConnection;
    property Port;
    property OnError;
  end;

  TSimpleTCPClient = class(TCustomSimpleSocket)
  private
    FConnected: Boolean;

    FOnConnected: TNotifyEvent;
    FOnDisconnected: TNotifyEvent;
    FOnRead: TSimpleTCPClientIOEvent;

    procedure SetConnected(Value: Boolean);
    procedure WMClientSelect(var Msg: TMessage); message WM_TCPASYNCSELECT;
  protected
    procedure SetHost(Value: String); override;
    procedure SetPort(Value: Word); override;
  public
    destructor Destroy; override;

    function Send(Buffer: PChar; Length: Integer): Integer; // bytes sent
  published
    property Connected: Boolean read FConnected write SetConnected;

    property OnConnected: TNotifyEvent read FOnConnected write FOnConnected;
    property OnDisconnected: TNotifyEvent read FOnDisconnected write FOnDisconnected;
    property OnRead: TSimpleTCPClientIOEvent read FOnRead write FOnRead;

    property AllowChangeHostAndPortOnConnection;
    property Host;
    property Port;
    property OnError;
  end;

procedure Register;

implementation

uses Forms;

const
  LineEnd: Char = #0;

{ TCustomSimpleSocket }
constructor TCustomSimpleSocket.Create(aOwner: TComponent);
var
  ErrorCode: Integer;
begin
  inherited Create(aOwner);

  FSocket := INVALID_SOCKET;
  WindowHandle := AllocateHWnd(WndProc);

  ErrorCode := WSAStartup($0101, WSAData);
  if ErrorCode <> 0 then
    raise Exception.Create('Can not start socket session');
end;

destructor TCustomSimpleSocket.Destroy;
var
  ErrorCode: Integer;
begin
  ErrorCode := WSACleanup;
  if ErrorCode <> 0 then
    raise Exception.Create('Can not clean socket session');

  DeallocateHWnd(WindowHandle);

  inherited Destroy;
end;

procedure TCustomSimpleSocket.SetHost(Value: String);
begin
end;

procedure TCustomSimpleSocket.SetPort(Value: Word);
begin
end;

procedure TCustomSimpleSocket.WndProc(var Msg: TMessage);
begin
  try
    Dispatch(Msg);
  except
    Application.HandleException(Self);
  end;
end;

procedure TCustomSimpleSocket.SetSocket(Value: TSocket);
var
  Len: Integer;
  SockAddrIn: TSockAddrIn;
begin
  FSocket := Value;

  Len := SizeOf(SockAddrIn);
  GetSockName(Value, SockAddrIn, Len);
  FHost := inet_ntoa(SockAddrIn.sin_addr);
  FPort := ntohs(SockAddrIn.sin_port);
end;

procedure TCustomSimpleSocket.SocketError(Socket: TSocket; ErrorCode: Integer);
var
  ErrorMsg: String;
begin
  case ErrorCode of
    WSAEINTR: ErrorMsg := 'Interrupted system call';
    WSAEBADF: ErrorMsg := 'Bad file number';
    WSAEACCES: ErrorMsg := 'Permission denied';
    WSAEFAULT: ErrorMsg := 'Bad address';
    WSAEINVAL: ErrorMsg := 'Invalid argument';
    WSAEMFILE: ErrorMsg := 'Too many open files';
    WSAEWOULDBLOCK: ErrorMsg := 'Operation would block';
    WSAEINPROGRESS: ErrorMsg := 'Operation now in progress';
    WSAEALREADY: ErrorMsg := 'Operation already in progress';
    WSAENOTSOCK: ErrorMsg := 'Socket operation on non-socket';
    WSAEDESTADDRREQ: ErrorMsg := 'Destination address required';
    WSAEMSGSIZE: ErrorMsg := 'Message too long';
    WSAEPROTOTYPE: ErrorMsg := 'Protocol wrong type for socket';
    WSAENOPROTOOPT: ErrorMsg := 'Protocol not available';
    WSAEPROTONOSUPPORT: ErrorMsg := 'Protocol not supported';
    WSAESOCKTNOSUPPORT: ErrorMsg := 'Socket type not supported';
    WSAEOPNOTSUPP: ErrorMsg := 'Operation not supported on socket';
    WSAEPFNOSUPPORT: ErrorMsg := 'Protocol family not supported';
    WSAEAFNOSUPPORT: ErrorMsg := 'Address family not supported by protocol family';
    WSAEADDRINUSE: ErrorMsg := 'Address already in use';
    WSAEADDRNOTAVAIL: ErrorMsg := 'Can''t assign requested address';
    WSAENETDOWN: ErrorMsg := 'Network is down';
    WSAENETUNREACH: ErrorMsg := 'Network is unreachable';
    WSAENETRESET: ErrorMsg := 'Network dropped connection on reset';
    WSAECONNABORTED: ErrorMsg := 'Software caused connection abort';
    WSAECONNRESET: ErrorMsg := 'Connection reset by peer';
    WSAENOBUFS: ErrorMsg := 'No buffer space available';
    WSAEISCONN: ErrorMsg := 'Socket is already connected';
    WSAENOTCONN: ErrorMsg := 'Socket is not connected';
    WSAESHUTDOWN: ErrorMsg := 'Can''t send after socket shutdown';
    WSAETOOMANYREFS: ErrorMsg := 'Too many references: can''t splice';
    WSAETIMEDOUT: ErrorMsg := 'Connection timed out';
    WSAECONNREFUSED: ErrorMsg := 'Connection refused';
    WSAELOOP: ErrorMsg := 'Too many levels of symbolic links';
    WSAENAMETOOLONG: ErrorMsg := 'File name too long';
    WSAEHOSTDOWN: ErrorMsg := 'Host is down';
    WSAEHOSTUNREACH: ErrorMsg := 'No route to host';
    WSAENOTEMPTY: ErrorMsg := 'Directory not empty';
    WSAEPROCLIM: ErrorMsg := 'Too many processes';
    WSAEUSERS: ErrorMsg := 'Too many users';
    WSAEDQUOT: ErrorMsg := 'Disk quota exceeded';
    WSAESTALE: ErrorMsg := 'Stale NFS file handle';
    WSAEREMOTE: ErrorMsg := 'Too many levels of remote in path';
    WSASYSNOTREADY: ErrorMsg := 'Network sub-system is unusable';
    WSAVERNOTSUPPORTED: ErrorMsg := 'WinSock DLL cannot support this application';
    WSANOTINITIALISED: ErrorMsg := 'WinSock not initialized';
    WSAHOST_NOT_FOUND: ErrorMsg := 'Host not found';
    WSATRY_AGAIN: ErrorMsg := 'Non-authoritative host not found';
    WSANO_RECOVERY: ErrorMsg := 'Non-recoverable error';
    WSANO_DATA: ErrorMsg := 'No Data';
    else ErrorMsg := 'Not a WinSock error ?';
  end;

  if Assigned(FOnError) then
    FOnError(Self, Socket, ErrorCode, ErrorMsg)
  else
   raise Exception.Create(ErrorMsg);
end;

function TCustomSimpleSocket.SendTo(Socket: TSocket; Buffer: PChar; Length: Integer): Integer; // bytes sent
var
  SLen: Integer;
begin
  Result := 0;
  if Socket <> INVALID_SOCKET then
   begin
    SLen := WinSock.Send(Socket, Buffer^, Length, 0);
    if SLen = SOCKET_ERROR then
      SocketError(Socket, WSAGetLastError);
    Result := SLen;
   end;
end;
{------------------------------------------------------------}

{ TSimpleTCPServer }
constructor TSimpleTCPServer.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);

  FConnections := TList.Create;
end;

destructor TSimpleTCPServer.Destroy;
begin
  Listen := False;
  FConnections.Free;

  inherited Destroy;
end;

procedure TSimpleTCPServer.WMServerSelect(var Msg: TMessage);
var
  i: Integer;
  SelectEvent, ErrorCode, Tmp: Integer;
  tmpSocket: TSocket;
  tmpTCPClient: TSimpleTCPClient;
  IsAccept: Boolean;
  Buf: Array[0..8192] of Char;
  BufLen, TotalSize: Integer;
  MemoryStream: TMemoryStream;
begin
  ErrorCode := WSAGetSelectError(Msg.LParam);
  if ErrorCode > WSABASEERR then
    SocketError(Msg.wParam, ErrorCode)
  else
   begin
    SelectEvent := WSAGetSelectEvent(Msg.lParam);
    case SelectEvent of
      FD_READ: begin
                tmpTCPClient := nil;
                tmpSocket := Msg.wParam;
                i := FConnections.Count;
                if i <> 0 then
                 for i := 0 to i - 1 do
                  begin
                   tmpTCPClient := FConnections[i];
                   if tmpTCPClient.FSocket = tmpSocket then Break;
                  end;

                if Assigned(FOnClientRead) then
                 begin
                  TotalSize := 0;
                  MemoryStream := TMemoryStream.Create;
                  repeat
                    FillChar(Buf, SizeOf(Buf), 0);
                    BufLen := recv(tmpSocket, Buf, SizeOf(Buf), 0);
                    if BufLen = SOCKET_ERROR then
                      SocketError(tmpSocket, WSAGetLastError);
                    MemoryStream.Write(Buf, BufLen);
                    inc(TotalSize, BufLen);
                  until BufLen < SizeOf(Buf);

                  MemoryStream.Write(LineEnd, SizeOf(LineEnd));
                  MemoryStream.Size := MemoryStream.Size - 1;
                  FOnClientRead(Self, tmpTCPClient, MemoryStream.Memory, TotalSize);
                  MemoryStream.Free;
                 end;
               end;
     FD_CLOSE: begin
                tmpTCPClient := nil;
                tmpSocket := Msg.wParam;
                i := FConnections.Count;
                if i <> 0 then
                 for i := 0 to i - 1 do
                  begin
                   tmpTCPClient := FConnections[i];
                   if tmpTCPClient.FSocket = tmpSocket then
                    begin
                     FConnections.Delete(i);
                     Break;
                    end;
                  end;

                if Assigned(tmpTCPClient) then
                 begin
                  if Assigned(FOnClientDisconnected) then
                    FOnClientDisconnected(Self, tmpTCPClient);
                  tmpTCPClient.Free;
                 end;
               end;
     FD_ACCEPT: begin
                 Tmp := SizeOf(SockAddrIn);

                 {$IFDEF DELPHI2}
                 tmpSocket := WinSock.Accept(FSocket, SockAddrIn, Tmp);
                 {$ELSE}
                 tmpSocket := WinSock.Accept(FSocket, @SockAddrIn, @Tmp);                 
                 {$ENDIF}
                 if tmpSocket = SOCKET_ERROR then
                   SocketError(tmpSocket, WSAGetLastError);

                 tmpTCPClient := TSimpleTCPClient.Create(nil);
                 tmpTCPClient.Socket := tmpSocket;
                 tmpTCPClient.FConnected := True;

                 if Assigned(FOnAccept) then
                  begin
                   IsAccept := True;
                   FOnAccept(Self, tmpTCPClient, IsAccept);
                   if IsAccept then
                    Connections.Add(tmpTCPClient)
                   else
                    tmpTCPClient.Free;
                  end
                 else
                  Connections.Add(tmpTCPClient);

                 if Assigned(FOnClientConnected) then
                  FOnClientConnected(Self, tmpTCPClient); 
                end;
    end;
  end;
end;

procedure TSimpleTCPServer.SendToAll(Buffer: PChar; Length: Integer);
var
  i: Integer;
  tmpTCPClient: TSimpleTCPClient;
  Socket: TSocket;
begin
  i := FConnections.Count;
  if i <> 0 then
   for i := 0 to i - 1 do
    begin
     tmpTCPClient := FConnections[i];
     Socket := tmpTCPClient.FSocket;
     SendTo(Socket, Buffer, Length);
    end;
end;

function TSimpleTCPServer.Send(Client: TSimpleTCPClient; Buffer: PChar; Length: Integer): Integer; // bytes sent
begin
  Result := SendTo(Client.FSocket, Buffer, Length);
end;

procedure TSimpleTCPServer.SetPort(Value: Word);
begin
  if not (csDesigning in ComponentState) then
   if FPort <> Value then
    if FListen then
     if FAllowChangeHostAndPortOnConnection then
      begin
       Listen := False;
       FPort := Value;
       Listen := True;
      end
     else
      raise Exception.Create('Can not change Port while listening')
    else FPort := Value
   else
  else FPort := Value;
end;

procedure TSimpleTCPServer.SetListen(Value: Boolean);
var
  ErrorCode: Integer;
  i: Integer;
  tmpTCPClient: TSimpleTCPClient;
begin
  if not (csDesigning in ComponentState) then
   if FListen <> Value then
    begin
     if Value then
      begin
       SockAddrIn.sin_family := AF_INET;
       SockAddrIn.sin_addr.s_addr := INADDR_ANY;
       SockAddrIn.sin_port := htons(FPort);
       PProtoEnt := GetProtoByName('tcp');

       FSocket := WinSock.Socket(PF_INET, SOCK_STREAM, PProtoEnt^.p_proto);
       if FSocket = SOCKET_ERROR then
        begin
         SocketError(INVALID_SOCKET, WSAGetLastError);
         Exit;
        end;

       ErrorCode := Bind(FSocket, SockAddrIn, SizeOf(SockAddrIn));
       if ErrorCode <> 0 then
        begin
         SocketError(FSocket, WSAGetLastError);
         Exit;
        end;

       ErrorCode := WinSock.Listen(FSocket, 5);
       if ErrorCode <> 0 then
        begin
         SocketError(FSocket, WSAGetLastError);
         Exit;
        end;

       ErrorCode := WSAASyncSelect(FSocket, WindowHandle, WM_TCPASYNCSELECT,
                                   FD_READ or FD_ACCEPT or FD_CLOSE);
       if ErrorCode <> 0 then
        begin
         SocketError(FSocket, WSAGetLastError);
         Exit;
        end;
      end
     else
      begin
       // Closing all connections first
       i := FConnections.Count;
       if i <> 0 then
        for i := i - 1 downto 0 do
         begin
          tmpTCPClient := FConnections[i];
          tmpTCPClient.Connected := False;
          FConnections.Delete(i);
         end;

       // Cancel listing

       WSAASyncSelect(FSocket, WindowHandle, WM_TCPASYNCSELECT, 0);
       Shutdown(FSocket, 2);

       ErrorCode := CloseSocket(FSocket);
       if ErrorCode <> 0 then
        begin
         SocketError(FSocket, WSAGetLastError);
         Exit;
        end;

       FSocket := INVALID_SOCKET;
      end;
      FListen := Value;
    end
   else
  else
   FListen := Value;
end;

function TSimpleTCPServer.GetLocalHostName: String;
var
  HostName: Array[0..128] of Char;
begin
  if GetHostName(HostName, 128) = 0 then
    Result := HostName
  else
    SocketError(FSocket, WSAGetLastError);
end;

function TSimpleTCPServer.GetLocalIP: String;
var
  SockAddrIn: TSockAddrIn;
  HostEnt: PHostEnt;
  HostName: Array[0..128] of Char;
begin
  if GetHostName(HostName, 128) = 0 then
   begin
    HostEnt:= GetHostByName(HostName);
    if HostEnt = nil then
      Result := ''
    else
     begin
      SockAddrIn.sin_addr.S_addr := LongInt(PLongInt(HostEnt^.h_addr_list^)^);
      Result := inet_ntoa(SockAddrIn.sin_addr);
     end;
   end
  else
   SocketError(FSocket, WSAGetLastError);
end;

procedure TSimpleTCPServer.SetNoneStr(Value: String); begin end;
{------------------------------------------------------------}


{ TSimpleTCPClient }
destructor TSimpleTCPClient.Destroy;
begin
  Connected := False;

  inherited Destroy;
end;

procedure TSimpleTCPClient.WMClientSelect(var Msg: TMessage);
var
  SelectEvent, ErrorCode: Integer;
  tmpSocket: TSocket;
  Buf: Array[0..8192] of Char;
  BufLen, TotalSize: Integer;
  MemoryStream: TMemoryStream;
begin
  ErrorCode := WSAGetSelectError(Msg.LParam);
  if ErrorCode > WSABASEERR then
    SocketError(Msg.wParam, ErrorCode)
  else
   begin
    SelectEvent := WSAGetSelectEvent(Msg.lParam);
    case SelectEvent of
      FD_READ: begin
                tmpSocket := Msg.wParam;
                if Assigned(FOnRead) then
                 begin
                  TotalSize := 0;
                  MemoryStream := TMemoryStream.Create;
                  repeat
                    FillChar(Buf, SizeOf(Buf), 0);
                    BufLen := recv(tmpSocket, Buf, SizeOf(Buf), 0);
                    if BufLen = SOCKET_ERROR then
                     begin
                      SocketError(tmpSocket, WSAGetLastError);
                      Exit;
                     end;
                    MemoryStream.Write(Buf, BufLen);
                    inc(TotalSize, BufLen);
                  until BufLen < SizeOf(Buf);
                  
                  MemoryStream.Write(LineEnd, SizeOf(LineEnd));
                  MemoryStream.Size := MemoryStream.Size - 1;
                  FOnRead(Self, MemoryStream.Memory, TotalSize);
                  MemoryStream.Free;
                 end;
               end;
     FD_CLOSE: begin
                Connected := False;
                if Assigned(FOnDisconnected) then
                  FOnDisconnected(Self);
               end;
     FD_CONNECT: begin
                  if Assigned(FOnConnected) then
                    FOnConnected(Self);
                 end;
    end;
  end;
end;

function TSimpleTCPClient.Send(Buffer: PChar; Length: Integer): Integer; // bytes sent
begin
  Result := SendTo(FSocket, Buffer, Length);
end;

procedure TSimpleTCPClient.SetConnected(Value: Boolean);
var
  lin: TLinger;
  linx: Array[0..3] of Char absolute lin;
  ErrorCode: Integer;
begin
  if not (csDesigning in ComponentState) then
   if FConnected <> Value then
    begin
     if Value then
      begin
       SockAddrIn.sin_family := AF_INET;
       SockAddrIn.sin_port := htons(FPort);
       SockAddrIn.sin_addr.s_addr := inet_addr(PChar(Host));
       if SockAddrIn.sin_addr.s_addr = -1 then
        begin
         HostEnt := GetHostByName(PChar(Host));
         if HostEnt = nil then
          begin
           SocketError(INVALID_SOCKET, WSAEFAULT);
           Exit;
          end;
         SockAddrIn.sin_addr.S_addr := LongInt(PLongInt(HostEnt^.h_addr_list^)^);
        end;
       PProtoEnt := GetProtoByName('tcp');
       FSocket := WinSock.Socket(PF_INET, SOCK_STREAM, PProtoEnt^.p_proto);
       if FSocket = SOCKET_ERROR then
        begin
         SocketError(INVALID_SOCKET, WSAGetLastError);
         Exit;
        end;

       ErrorCode := WSAASyncSelect(FSocket, WindowHandle, WM_TCPASYNCSELECT,
                                   FD_READ or FD_CONNECT or FD_CLOSE);
       if ErrorCode <> 0 then
        begin
         SocketError(FSocket, WSAGetLastError);
         Exit;
        end;

       ErrorCode := WinSock.Connect(FSocket, SockAddrIn, SizeOf(SockAddrIn));
       if ErrorCode <> 0 then
        begin
         ErrorCode := WSAGetLastError;
         if ErrorCode <> WSAEWOULDBLOCK then
          begin
           SocketError(FSocket, WSAGetLastError);
           Exit;
          end;
        end;
      end
     else
      begin
       WSAASyncSelect(FSocket, WindowHandle, WM_TCPASYNCSELECT, 0);
       Shutdown(FSocket, 2);
       lin.l_onoff := 1;
       lin.l_linger := 0;
       SetSockOpt(FSocket, SOL_SOCKET, SO_LINGER, linx, SizeOf(Lin));

       ErrorCode := CloseSocket(FSocket);
       if ErrorCode <> 0 then
        begin
         SocketError(FSocket, WSAGetLastError);
         Exit;
        end;

       FSocket := INVALID_SOCKET;
      end;

     FConnected := Value;
    end
   else
  else
   if Value then
    raise Exception.Create('Can not connect at design-time');
end;

procedure TSimpleTCPClient.SetHost(Value: String);
begin
  if not (csDesigning in ComponentState) then
   if FHost <> Value then
    if FConnected then
     if FAllowChangeHostAndPortOnConnection then
      begin
       Connected := False;
       FHost := Value;
       Connected := True;
      end
     else
      raise Exception.Create('Can not change Host while connected')
    else
     FHost := Value
   else
  else FHost := Value;   
end;

procedure TSimpleTCPClient.SetPort(Value: Word);
begin
  if not (csDesigning in ComponentState) then
   if FPort <> Value then
    if FConnected then
     if FAllowChangeHostAndPortOnConnection then
      begin
       Connected := False;
       FPort := Value;
       Connected := True;
      end
     else
      raise Exception.Create('Can not change Port while connected')
    else
     FPort := Value
   else
  else
   FPort := Value;   
end;

procedure Register;
begin
  RegisterComponents('Samples', [TSimpleTCPServer, TSimpleTCPClient]);
end;

end.
